package org.myorg.ddlvisualizer;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;
import org.netbeans.api.languages.ASTNode;
import org.netbeans.api.visual.action.ActionFactory;
import org.netbeans.api.visual.vmd.VMDGraphScene;
import org.netbeans.api.visual.vmd.VMDNodeWidget;
import org.netbeans.api.visual.vmd.VMDPinWidget;

/** 
 * 
 * @author Damir Tesanovic (based on Anton Epple's DBGraphScene) 
 */
public class DDLVisualScene extends VMDGraphScene {

    private static int edgeID = 1;

    public DDLVisualScene() {
        this.getActions().addAction(ActionFactory.createZoomAction());
        this.getActions().addAction(ActionFactory.createPanAction());

    }

    public void setASTNode(ASTNode node) {
        for (ASTNode table : getNodes(node, "CreateTable", false)) {
            String tableName = table.getNode("TableName").getTokenTypeIdentifier("identifier");
            createNode(this, (int) (Math.random() * 800), (int) (Math.random() * 800), tableName, "Table", null);

            for (ASTNode col : getNodes(table, "ColumnDef", true)) {
                String columnName = col.getNode("ColumnName").getTokenTypeIdentifier("identifier");
                createPin(this, tableName, tableName + ":" + columnName, columnName, columnName);
            }

            for (ASTNode foreignKeyClause : getNodes(table, "ForeignKeyClause", true)) {
                ArrayList<String> cols = new ArrayList<String>();
                for (ASTNode col : getNodes(foreignKeyClause, "ColumnName", false)) {
                    cols.add(col.getTokenTypeIdentifier("identifier"));
                }

                String refTableName = foreignKeyClause.getNode("ReferencesClause").getNode("TableName").getTokenTypeIdentifier("identifier");
                ArrayList<String> refCols = new ArrayList<String>();
                for (ASTNode refCol : getNodes(foreignKeyClause.getNode("ReferencesClause"), "ColumnName", false)) {
                    refCols.add(refCol.getTokenTypeIdentifier("identifier"));
                }

                for (int i = 0; i < cols.size(); i++) {
                    createEdge(this, refTableName + ":" + refCols.get(i), tableName + ":" + cols.get(i));
                }

            }
        }

        this.moveTo(null);

    }

    private ArrayList<ASTNode> getNodes(ASTNode root, String nt, boolean deep) {
        ArrayList<ASTNode> nodes = new ArrayList<ASTNode>();
        fillNodes(root, nodes, nt, deep);
        return nodes;
    }

    private void fillNodes(ASTNode node, ArrayList<ASTNode> out, String nt, boolean deep) {
        Iterator it = node.getChildren().iterator();
        while (it.hasNext()) {
            Object elem = it.next();
            if (elem instanceof ASTNode) {
                if (((ASTNode) elem).getNT().equals(nt)) {
                    out.add((ASTNode) elem);
                }
                if (deep) {
                    fillNodes((ASTNode) elem, out, nt, deep);
                }
            }
        }
    }

    private static String createNode(VMDGraphScene scene, int x, int y, String name, String type, java.util.List<Image> glyphs) {
        String nodeID = name;
        if (!scene.getNodes().contains(nodeID)) {
            VMDNodeWidget widget = (VMDNodeWidget) scene.addNode(nodeID);
            widget.setPreferredLocation(new Point(x, y));
            widget.setNodeProperties(null, name, type, glyphs);
        }
        return nodeID;
    }

    private static void createPin(VMDGraphScene scene, String nodeID, String pinID, String name, String type) {
        if (!scene.getPins().contains(pinID)) {
            ((VMDPinWidget) scene.addPin(nodeID, pinID)).setProperties(name, null);
        }
    }

    private static void createEdge(VMDGraphScene scene, String sourcePinID, String targetPinID) {
        String edgeID = "edge" + DDLVisualScene.edgeID++;
        scene.addEdge(edgeID);
        System.out.println("createEdge " + sourcePinID + "<->" + targetPinID);
        scene.setEdgeSource(edgeID, sourcePinID);
        scene.setEdgeTarget(edgeID, targetPinID);
    }

    private void moveTo(Point point) {
        int index = 0;
        for (String node : getNodes()) {
            getSceneAnimator().animatePreferredLocation(findWidget(node), point != null ? point : new Point(++index * 100, index * 100));
        }
    }
} 
						